#include <osg/Geode>
#include <osg/ShapeDrawable>
#include <osg/Geometry>
#include <osg/MatrixTransform>
#include <osg/PositionAttitudeTransform>
#include <osgViewer/Viewer>
#include <osgGA/TrackballManipulator>
#include <osgDB/ReadFile>
#include <osgDB/WriteFile>
#include <iostream>

#include "Flipbook.h"

osg::ref_ptr<osg::Node> CreateLight()
{
    osg::ref_ptr<osg::Group> pGroup = new osg::Group;

	// create light
	osg::ref_ptr<osg::Light> pLight = new osg::Light;
    pLight->setLightNum( 2 );
	pLight->setAmbient(osg::Vec4(.1f, .1f, .1f, 1.f));
	pLight->setDiffuse(osg::Vec4(1.f, 0.f, 0.f, 1.f));		// red light
	pLight->setSpecular(osg::Vec4(.8f, .8f, .8f, 1.f));
	pLight->setPosition(osg::Vec4(0.f, 0.f, 0.f, 1.f));	// positional light

	// create light source
	osg::ref_ptr<osg::LightSource> pLightSource = new osg::LightSource;
	pLightSource->setLight(pLight.get());

	osg::ref_ptr<osg::MatrixTransform> pMatTrans = new osg::MatrixTransform;
	osg::Matrix m;
	m.makeTranslate(osg::Vec3(0.f, -1.f, 0.f));
	pMatTrans->setMatrix(m);
	pMatTrans->addChild(pLightSource.get());

	// set stateset
	pGroup->addChild(pMatTrans.get());
	pGroup->getOrCreateStateSet()->setMode( GL_LIGHT2, osg::StateAttribute::ON );

	return pGroup.get();
}

osg::ref_ptr<osg::Node> CreateSceneGraph()
{
	osg::ref_ptr<osg::Group> pGroup = new osg::Group;

	pGroup->addChild(CreateLight().get());

	osg::ref_ptr<Flipbook> node = new Flipbook;
	if (node.valid()) {
		node->LoadModels("angel", "iv", 5);
		node->setUpdateCallback(new FlipBookAnimationCallback(ANIMATION_RATE));
		pGroup->addChild(node.get());
	}
	
	return pGroup.get();
}

int main(int argc, char **argv)
{
    // use an ArgumentParser object to manage the program arguments
    osg::DisplaySettings::instance()->setMinimumNumAlphaBits(8);
   
    // construct the viewer
    osg::ref_ptr<osgViewer::Viewer> viewer = new osgViewer::Viewer;

	// make the viewer create a 512x512 window and position it at 32, 32
    viewer->setUpViewInWindow( 32, 32, 512, 512 );

    // add the node data to the viewer
    viewer->setSceneData(CreateSceneGraph().get());

	// execute main loop
	viewer->run();

	return 0;
}
